@@ -1,8 +1,9 @@
 import django
 from django.contrib import admin
 
-from .models import EmailConfirmation, EmailAddress
+from . import app_settings
 from .adapter import get_adapter
+from .models import EmailAddress, EmailConfirmation
 
 
 class EmailAddressAdmin(admin.ModelAdmin):
@@ -17,7 +18,7 @@ class EmailAddressAdmin(admin.ModelAdmin):
             self.search_fields = self.get_search_fields(None)
 
     def get_search_fields(self, request):
-        base_fields = get_adapter().get_user_search_fields()
+        base_fields = get_adapter(request).get_user_search_fields()
         return ['email'] + list(map(lambda a: 'user__' + a, base_fields))
 
 
@@ -27,5 +28,6 @@ class EmailConfirmationAdmin(admin.ModelAdmin):
     raw_id_fields = ('email_address',)
 
 
-admin.site.register(EmailConfirmation, EmailConfirmationAdmin)
+if not app_settings.EMAIL_CONFIRMATION_HMAC:
+    admin.site.register(EmailConfirmation, EmailConfirmationAdmin)
 admin.site.register(EmailAddress, EmailAddressAdmin)
