@@ -1,62 +1,74 @@
-from django.core.urlresolvers import reverse, reverse_lazy
-from django.http import (HttpResponseRedirect, Http404,
-                         HttpResponsePermanentRedirect)
-from django.views.generic.base import TemplateResponseMixin, View, TemplateView
-from django.views.generic.edit import FormView
 from django.contrib import messages
 from django.contrib.auth.decorators import login_required
-from django.contrib.auth import logout as auth_logout
+from django.contrib.sites.shortcuts import get_current_site
+from django.http import (
+    Http404,
+    HttpResponsePermanentRedirect,
+    HttpResponseRedirect,
+)
 from django.shortcuts import redirect
-from django.views.decorators.debug import sensitive_post_parameters
 from django.utils.decorators import method_decorator
+from django.views.decorators.debug import sensitive_post_parameters
+from django.views.generic.base import TemplateResponseMixin, TemplateView, View
+from django.views.generic.edit import FormView
 
+from . import app_settings, signals
+from ..compat import is_anonymous, is_authenticated, reverse, reverse_lazy
 from ..exceptions import ImmediateHttpResponse
-from ..utils import get_form_class, get_request_param, get_current_site
-
-from .utils import (get_next_redirect_url, complete_signup,
-                    get_login_redirect_url, perform_login,
-                    passthrough_next_redirect_url)
-from .forms import AddEmailForm, ChangePasswordForm
-from .forms import LoginForm, ResetPasswordKeyForm
-from .forms import ResetPasswordForm, SetPasswordForm, SignupForm, UserTokenForm
-from .utils import sync_user_email_addresses
-from .models import EmailAddress, EmailConfirmation
-
-from . import signals
-from . import app_settings
-
+from ..utils import get_form_class, get_request_param
 from .adapter import get_adapter
+from .forms import (
+    AddEmailForm,
+    ChangePasswordForm,
+    LoginForm,
+    ResetPasswordForm,
+    ResetPasswordKeyForm,
+    SetPasswordForm,
+    SignupForm,
+    UserTokenForm,
+)
+from .models import EmailAddress, EmailConfirmation, EmailConfirmationHMAC
+from .utils import (
+    complete_signup,
+    get_login_redirect_url,
+    get_next_redirect_url,
+    logout_on_password_change,
+    passthrough_next_redirect_url,
+    perform_login,
+    sync_user_email_addresses,
+    url_str_to_user_pk,
+)
 
-try:
-    from django.contrib.auth import update_session_auth_hash
-except ImportError:
-    update_session_auth_hash = None
+
+INTERNAL_RESET_URL_KEY = "set-password"
+INTERNAL_RESET_SESSION_KEY = "_password_reset_key"
 
 
 sensitive_post_parameters_m = method_decorator(
     sensitive_post_parameters('password', 'password1', 'password2'))
 
 
-def _ajax_response(request, response, form=None):
+def _ajax_response(request, response, form=None, data=None):
     if request.is_ajax():
-        if (isinstance(response, HttpResponseRedirect)
-                or isinstance(response, HttpResponsePermanentRedirect)):
+        if (isinstance(response, HttpResponseRedirect) or isinstance(
+                response, HttpResponsePermanentRedirect)):
             redirect_to = response['Location']
         else:
             redirect_to = None
-        response = get_adapter().ajax_response(request,
-                                               response,
-                                               form=form,
-                                               redirect_to=redirect_to)
+        response = get_adapter(request).ajax_response(
+            request,
+            response,
+            form=form,
+            data=data,
+            redirect_to=redirect_to)
     return response
 
 
 class RedirectAuthenticatedUserMixin(object):
+
     def dispatch(self, request, *args, **kwargs):
-        # WORKAROUND: https://code.djangoproject.com/ticket/19316
-        self.request = request
-        # (end WORKAROUND)
-        if request.user.is_authenticated():
+        if is_authenticated(request.user) and \
+                app_settings.AUTHENTICATED_LOGIN_REDIRECTS:
             redirect_to = self.get_authenticated_redirect_url()
             response = HttpResponseRedirect(redirect_to)
             return _ajax_response(request, response)
@@ -76,6 +88,13 @@ class RedirectAuthenticatedUserMixin(object):
 
 class AjaxCapableProcessFormViewMixin(object):
 
+    def get(self, request, *args, **kwargs):
+        response = super(AjaxCapableProcessFormViewMixin, self).get(
+            request, *args, **kwargs)
+        form = self.get_form()
+        return _ajax_response(
+            self.request, response, form=form, data=self._get_ajax_data_if())
+
     def post(self, request, *args, **kwargs):
         form_class = self.get_form_class()
         form = self.get_form(form_class)
@@ -83,14 +102,29 @@ class AjaxCapableProcessFormViewMixin(object):
             response = self.form_valid(form)
         else:
             response = self.form_invalid(form)
-        return _ajax_response(self.request, response, form=form)
+        return _ajax_response(
+            self.request, response, form=form, data=self._get_ajax_data_if())
+
+    def get_form(self, form_class=None):
+        form = getattr(self, '_cached_form', None)
+        if form is None:
+            form = super(AjaxCapableProcessFormViewMixin, self).get_form(
+                form_class)
+            self._cached_form = form
+        return form
+
+    def _get_ajax_data_if(self):
+        return self.get_ajax_data() if self.request.is_ajax() else None
+
+    def get_ajax_data(self):
+        return None
 
 
 class LoginView(RedirectAuthenticatedUserMixin,
                 AjaxCapableProcessFormViewMixin,
                 FormView):
     form_class = LoginForm
-    template_name = "account/login.html"
+    template_name = "account/login." + app_settings.TEMPLATE_EXTENSION
     success_url = None
     redirect_field_name = "next"
 
@@ -98,6 +132,11 @@ class LoginView(RedirectAuthenticatedUserMixin,
     def dispatch(self, request, *args, **kwargs):
         return super(LoginView, self).dispatch(request, *args, **kwargs)
 
+    def get_form_kwargs(self):
+        kwargs = super(LoginView, self).get_form_kwargs()
+        kwargs['request'] = self.request
+        return kwargs
+
     def get_form_class(self):
         return get_form_class(app_settings.FORMS, 'login', self.form_class)
 
@@ -110,9 +149,9 @@ class LoginView(RedirectAuthenticatedUserMixin,
 
     def get_success_url(self):
         # Explicitly passed ?next= URL takes precedence
-        ret = (get_next_redirect_url(self.request,
-                                     self.redirect_field_name)
-               or self.success_url)
+        ret = (get_next_redirect_url(
+            self.request,
+            self.redirect_field_name) or self.success_url)
         return ret
 
     def get_context_data(self, **kwargs):
@@ -130,16 +169,15 @@ class LoginView(RedirectAuthenticatedUserMixin,
                     "redirect_field_value": redirect_field_value})
         return ret
 
+
 login = LoginView.as_view()
 
 
 class CloseableSignupMixin(object):
-    template_name_signup_closed = "account/signup_closed.html"
+    template_name_signup_closed = (
+        "account/signup_closed." + app_settings.TEMPLATE_EXTENSION)
 
     def dispatch(self, request, *args, **kwargs):
-        # WORKAROUND: https://code.djangoproject.com/ticket/19316
-        self.request = request
-        # (end WORKAROUND)
         try:
             if not self.is_open():
                 return self.closed()
@@ -150,7 +188,7 @@ class CloseableSignupMixin(object):
                                                           **kwargs)
 
     def is_open(self):
-        return get_adapter().is_open_for_signup(self.request)
+        return get_adapter(self.request).is_open_for_signup(self.request)
 
     def closed(self):
         response_kwargs = {
@@ -162,7 +200,7 @@ class CloseableSignupMixin(object):
 
 class SignupView(RedirectAuthenticatedUserMixin, CloseableSignupMixin,
                  AjaxCapableProcessFormViewMixin, FormView):
-    template_name = "account/signup.html"
+    template_name = "account/signup." + app_settings.TEMPLATE_EXTENSION
     form_class = SignupForm
     redirect_field_name = "next"
     success_url = None
@@ -176,22 +214,34 @@ class SignupView(RedirectAuthenticatedUserMixin, CloseableSignupMixin,
 
     def get_success_url(self):
         # Explicitly passed ?next= URL takes precedence
-        ret = (get_next_redirect_url(self.request,
-                                     self.redirect_field_name)
-               or self.success_url)
+        ret = (
+            get_next_redirect_url(
+                self.request,
+                self.redirect_field_name) or self.success_url)
         return ret
 
     def form_valid(self, form):
-        user = form.save(self.request)
-        return complete_signup(self.request, user,
-                               app_settings.EMAIL_VERIFICATION,
-                               self.get_success_url())
+        # By assigning the User to a property on the view, we allow subclasses
+        # of SignupView to access the newly created User instance
+        self.user = form.save(self.request)
+        try:
+            return complete_signup(
+                self.request, self.user,
+                app_settings.EMAIL_VERIFICATION,
+                self.get_success_url())
+        except ImmediateHttpResponse as e:
+            return e.response
 
     def get_context_data(self, **kwargs):
-        form = kwargs['form']
-        form.fields["email"].initial = self.request.session \
-            .get('account_verified_email', None)
         ret = super(SignupView, self).get_context_data(**kwargs)
+        form = ret['form']
+        email = self.request.session.get('account_verified_email')
+        if email:
+            email_keys = ['email']
+            if app_settings.SIGNUP_EMAIL_ENTER_TWICE:
+                email_keys.append('email2')
+            for email_key in email_keys:
+                form.fields[email_key].initial = email
         login_url = passthrough_next_redirect_url(self.request,
                                                   reverse("account_login"),
                                                   self.redirect_field_name)
@@ -203,16 +253,13 @@ class SignupView(RedirectAuthenticatedUserMixin, CloseableSignupMixin,
                     "redirect_field_value": redirect_field_value})
         return ret
 
+
 signup = SignupView.as_view()
 
 
 class ConfirmEmailView(TemplateResponseMixin, View):
 
-    def get_template_names(self):
-        if self.request.method == 'POST':
-            return ["account/email_confirmed.html"]
-        else:
-            return ["account/email_confirm.html"]
+    template_name = "account/email_confirm." + app_settings.TEMPLATE_EXTENSION
 
     def get(self, *args, **kwargs):
         try:
@@ -227,10 +274,11 @@ class ConfirmEmailView(TemplateResponseMixin, View):
     def post(self, *args, **kwargs):
         self.object = confirmation = self.get_object()
         confirmation.confirm(self.request)
-        get_adapter().add_message(self.request,
-                                  messages.SUCCESS,
-                                  'account/messages/email_confirmed.txt',
-                                  {'email': confirmation.email_address.email})
+        get_adapter(self.request).add_message(
+            self.request,
+            messages.SUCCESS,
+            'account/messages/email_confirmed.txt',
+            {'email': confirmation.email_address.email})
         if app_settings.LOGIN_ON_EMAIL_CONFIRMATION:
             resp = self.login_on_confirm(confirmation)
             if resp is not None:
@@ -270,22 +318,32 @@ class ConfirmEmailView(TemplateResponseMixin, View):
         may not 100% work in case the user closes the browser (and the
         session gets lost), but at least we're secure.
         """
-        user_pk = self.request.session.pop('account_user', None)
+        user_pk = None
+        user_pk_str = get_adapter(self.request).unstash_user(self.request)
+        if user_pk_str:
+            user_pk = url_str_to_user_pk(user_pk_str)
         user = confirmation.email_address.user
-        if user_pk == user.pk and self.request.user.is_anonymous():
+        if user_pk == user.pk and is_anonymous(self.request.user):
             return perform_login(self.request,
                                  user,
-                                 app_settings.EmailVerificationMethod.NONE)
+                                 app_settings.EmailVerificationMethod.NONE,
+                                 # passed as callable, as this method
+                                 # depends on the authenticated state
+                                 redirect_url=self.get_redirect_url)
 
         return None
 
     def get_object(self, queryset=None):
-        if queryset is None:
-            queryset = self.get_queryset()
-        try:
-            return queryset.get(key=self.kwargs["key"].lower())
-        except EmailConfirmation.DoesNotExist:
-            raise Http404()
+        key = self.kwargs['key']
+        emailconfirmation = EmailConfirmationHMAC.from_key(key)
+        if not emailconfirmation:
+            if queryset is None:
+                queryset = self.get_queryset()
+            try:
+                emailconfirmation = queryset.get(key=key.lower())
+            except EmailConfirmation.DoesNotExist:
+                raise Http404()
+        return emailconfirmation
 
     def get_queryset(self):
         qs = EmailConfirmation.objects.all_valid()
@@ -295,16 +353,20 @@ class ConfirmEmailView(TemplateResponseMixin, View):
     def get_context_data(self, **kwargs):
         ctx = kwargs
         ctx["confirmation"] = self.object
+        site = get_current_site(self.request)
+        ctx.update({'site': site})
         return ctx
 
     def get_redirect_url(self):
-        return get_adapter().get_email_confirmation_redirect_url(self.request)
+        return get_adapter(self.request).get_email_confirmation_redirect_url(
+            self.request)
+
 
 confirm_email = ConfirmEmailView.as_view()
 
 
 class EmailView(AjaxCapableProcessFormViewMixin, FormView):
-    template_name = "account/email.html"
+    template_name = "account/email." + app_settings.TEMPLATE_EXTENSION
     form_class = AddEmailForm
     success_url = reverse_lazy('account_email')
 
@@ -322,11 +384,12 @@ class EmailView(AjaxCapableProcessFormViewMixin, FormView):
 
     def form_valid(self, form):
         email_address = form.save(self.request)
-        get_adapter().add_message(self.request,
-                                  messages.INFO,
-                                  'account/messages/'
-                                  'email_confirmation_sent.txt',
-                                  {'email': form.cleaned_data["email"]})
+        get_adapter(self.request).add_message(
+            self.request,
+            messages.INFO,
+            'account/messages/'
+            'email_confirmation_sent.txt',
+            {'email': form.cleaned_data["email"]})
         signals.email_added.send(sender=self.request.user.__class__,
                                  request=self.request,
                                  user=self.request.user,
@@ -344,14 +407,14 @@ class EmailView(AjaxCapableProcessFormViewMixin, FormView):
                 res = self._action_remove(request)
             elif "action_primary" in request.POST:
                 res = self._action_primary(request)
-            res = res or HttpResponseRedirect(reverse('account_email'))
+            res = res or HttpResponseRedirect(self.success_url)
             # Given that we bypassed AjaxCapableProcessFormViewMixin,
             # we'll have to call invoke it manually...
-            res = _ajax_response(request, res)
+            res = _ajax_response(request, res, data=self._get_ajax_data_if())
         else:
             # No email address selected
-            res = HttpResponseRedirect(reverse('account_email'))
-            res = _ajax_response(request, res)
+            res = HttpResponseRedirect(self.success_url)
+            res = _ajax_response(request, res, data=self._get_ajax_data_if())
         return res
 
     def _action_send(self, request, *args, **kwargs):
@@ -361,11 +424,12 @@ class EmailView(AjaxCapableProcessFormViewMixin, FormView):
                 user=request.user,
                 email=email,
             )
-            get_adapter().add_message(request,
-                                      messages.INFO,
-                                      'account/messages/'
-                                      'email_confirmation_sent.txt',
-                                      {'email': email})
+            get_adapter(request).add_message(
+                request,
+                messages.INFO,
+                'account/messages/'
+                'email_confirmation_sent.txt',
+                {'email': email})
             email_address.send_confirmation(request)
             return HttpResponseRedirect(self.get_success_url())
         except EmailAddress.DoesNotExist:
@@ -379,21 +443,23 @@ class EmailView(AjaxCapableProcessFormViewMixin, FormView):
                 email=email
             )
             if email_address.primary:
-                get_adapter().add_message(request,
-                                          messages.ERROR,
-                                          'account/messages/'
-                                          'cannot_delete_primary_email.txt',
-                                          {"email": email})
+                get_adapter(request).add_message(
+                    request,
+                    messages.ERROR,
+                    'account/messages/'
+                    'cannot_delete_primary_email.txt',
+                    {"email": email})
             else:
                 email_address.delete()
                 signals.email_removed.send(sender=request.user.__class__,
                                            request=request,
                                            user=request.user,
                                            email_address=email_address)
-                get_adapter().add_message(request,
-                                          messages.SUCCESS,
-                                          'account/messages/email_deleted.txt',
-                                          {"email": email})
+                get_adapter(request).add_message(
+                    request,
+                    messages.SUCCESS,
+                    'account/messages/email_deleted.txt',
+                    {"email": email})
                 return HttpResponseRedirect(self.get_success_url())
         except EmailAddress.DoesNotExist:
             pass
@@ -401,9 +467,9 @@ class EmailView(AjaxCapableProcessFormViewMixin, FormView):
     def _action_primary(self, request, *args, **kwargs):
         email = request.POST["email"]
         try:
-            email_address = EmailAddress.objects.get(
+            email_address = EmailAddress.objects.get_for_user(
                 user=request.user,
-                email=email,
+                email=email
             )
             # Not primary=True -- Slightly different variation, don't
             # require verified unless moving from a verified
@@ -412,10 +478,11 @@ class EmailView(AjaxCapableProcessFormViewMixin, FormView):
             if not email_address.verified and \
                     EmailAddress.objects.filter(user=request.user,
                                                 verified=True).exists():
-                get_adapter().add_message(request,
-                                          messages.ERROR,
-                                          'account/messages/'
-                                          'unverified_primary_email.txt')
+                get_adapter(request).add_message(
+                    request,
+                    messages.ERROR,
+                    'account/messages/'
+                    'unverified_primary_email.txt')
             else:
                 # Sending the old primary address to the signal
                 # adds a db query.
@@ -425,10 +492,10 @@ class EmailView(AjaxCapableProcessFormViewMixin, FormView):
                 except EmailAddress.DoesNotExist:
                     from_email_address = None
                 email_address.set_as_primary()
-                get_adapter() \
-                    .add_message(request,
-                                 messages.SUCCESS,
-                                 'account/messages/primary_email_set.txt')
+                get_adapter(request).add_message(
+                    request,
+                    messages.SUCCESS,
+                    'account/messages/primary_email_set.txt')
                 signals.email_changed \
                     .send(sender=request.user.__class__,
                           request=request,
@@ -446,11 +513,23 @@ class EmailView(AjaxCapableProcessFormViewMixin, FormView):
         # (end NOTE)
         return ret
 
+    def get_ajax_data(self):
+        data = []
+        for emailaddress in self.request.user.emailaddress_set.all():
+            data.append({
+                'email': emailaddress.email,
+                'verified': emailaddress.verified,
+                'primary': emailaddress.primary,
+            })
+        return data
+
+
 email = login_required(EmailView.as_view())
 
 
 class PasswordChangeView(AjaxCapableProcessFormViewMixin, FormView):
-    template_name = "account/password_change.html"
+    template_name = (
+        "account/password_change." + app_settings.TEMPLATE_EXTENSION)
     form_class = ChangePasswordForm
     success_url = reverse_lazy("account_change_password")
 
@@ -461,10 +540,14 @@ class PasswordChangeView(AjaxCapableProcessFormViewMixin, FormView):
 
     @sensitive_post_parameters_m
     def dispatch(self, request, *args, **kwargs):
-        if not request.user.has_usable_password():
+        return super(PasswordChangeView, self).dispatch(
+            request, *args, **kwargs)
+
+    def render_to_response(self, context, **response_kwargs):
+        if not self.request.user.has_usable_password():
             return HttpResponseRedirect(reverse('account_set_password'))
-        return super(PasswordChangeView, self).dispatch(request, *args,
-                                                        **kwargs)
+        return super(PasswordChangeView, self).render_to_response(
+            context, **response_kwargs)
 
     def get_form_kwargs(self):
         kwargs = super(PasswordChangeView, self).get_form_kwargs()
@@ -473,12 +556,11 @@ class PasswordChangeView(AjaxCapableProcessFormViewMixin, FormView):
 
     def form_valid(self, form):
         form.save()
-        if (update_session_auth_hash is not None and 
-            not app_settings.LOGOUT_ON_PASSWORD_CHANGE):
-            update_session_auth_hash(self.request, form.user)
-        get_adapter().add_message(self.request,
-                                  messages.SUCCESS,
-                                  'account/messages/password_changed.txt')
+        logout_on_password_change(self.request, form.user)
+        get_adapter(self.request).add_message(
+            self.request,
+            messages.SUCCESS,
+            'account/messages/password_changed.txt')
         signals.password_changed.send(sender=self.request.user.__class__,
                                       request=self.request,
                                       user=self.request.user)
@@ -491,11 +573,12 @@ class PasswordChangeView(AjaxCapableProcessFormViewMixin, FormView):
         # (end NOTE)
         return ret
 
+
 password_change = login_required(PasswordChangeView.as_view())
 
 
 class PasswordSetView(AjaxCapableProcessFormViewMixin, FormView):
-    template_name = "account/password_set.html"
+    template_name = "account/password_set." + app_settings.TEMPLATE_EXTENSION
     form_class = SetPasswordForm
     success_url = reverse_lazy("account_set_password")
 
@@ -506,10 +589,14 @@ class PasswordSetView(AjaxCapableProcessFormViewMixin, FormView):
 
     @sensitive_post_parameters_m
     def dispatch(self, request, *args, **kwargs):
-        if request.user.has_usable_password():
+        if self.request.user.has_usable_password():
             return HttpResponseRedirect(reverse('account_change_password'))
         return super(PasswordSetView, self).dispatch(request, *args, **kwargs)
 
+    def render_to_response(self, context, **response_kwargs):
+        return super(PasswordSetView, self).render_to_response(
+            context, **response_kwargs)
+
     def get_form_kwargs(self):
         kwargs = super(PasswordSetView, self).get_form_kwargs()
         kwargs["user"] = self.request.user
@@ -517,9 +604,11 @@ class PasswordSetView(AjaxCapableProcessFormViewMixin, FormView):
 
     def form_valid(self, form):
         form.save()
-        get_adapter().add_message(self.request,
-                                  messages.SUCCESS,
-                                  'account/messages/password_set.txt')
+        logout_on_password_change(self.request, form.user)
+        get_adapter(self.request).add_message(
+            self.request,
+            messages.SUCCESS,
+            'account/messages/password_set.txt')
         signals.password_set.send(sender=self.request.user.__class__,
                                   request=self.request, user=self.request.user)
         return super(PasswordSetView, self).form_valid(form)
@@ -531,13 +620,15 @@ class PasswordSetView(AjaxCapableProcessFormViewMixin, FormView):
         # (end NOTE)
         return ret
 
+
 password_set = login_required(PasswordSetView.as_view())
 
 
 class PasswordResetView(AjaxCapableProcessFormViewMixin, FormView):
-    template_name = "account/password_reset.html"
+    template_name = "account/password_reset." + app_settings.TEMPLATE_EXTENSION
     form_class = ResetPasswordForm
     success_url = reverse_lazy("account_reset_password_done")
+    redirect_field_name = "next"
 
     def get_form_class(self):
         return get_form_class(app_settings.FORMS,
@@ -550,22 +641,30 @@ class PasswordResetView(AjaxCapableProcessFormViewMixin, FormView):
 
     def get_context_data(self, **kwargs):
         ret = super(PasswordResetView, self).get_context_data(**kwargs)
+        login_url = passthrough_next_redirect_url(self.request,
+                                                  reverse("account_login"),
+                                                  self.redirect_field_name)
         # NOTE: For backwards compatibility
         ret['password_reset_form'] = ret.get('form')
         # (end NOTE)
+        ret.update({"login_url": login_url})
         return ret
 
+
 password_reset = PasswordResetView.as_view()
 
 
 class PasswordResetDoneView(TemplateView):
-    template_name = "account/password_reset_done.html"
+    template_name = (
+        "account/password_reset_done." + app_settings.TEMPLATE_EXTENSION)
+
 
 password_reset_done = PasswordResetDoneView.as_view()
 
 
 class PasswordResetFromKeyView(AjaxCapableProcessFormViewMixin, FormView):
-    template_name = "account/password_reset_from_key.html"
+    template_name = (
+        "account/password_reset_from_key." + app_settings.TEMPLATE_EXTENSION)
     form_class = ResetPasswordKeyForm
     success_url = reverse_lazy("account_reset_password_from_key_done")
 
@@ -578,20 +677,43 @@ class PasswordResetFromKeyView(AjaxCapableProcessFormViewMixin, FormView):
         self.request = request
         self.key = key
 
-        # (Ab)using forms here to be able to handle errors in XHR #890
-        token_form = UserTokenForm(data={'uidb36': uidb36, 'key': key})
-
-        if not token_form.is_valid():
-            response = self.render_to_response(
-                self.get_context_data(token_fail=True)
-            )
-            return _ajax_response(self.request, response, form=token_form)
+        if self.key == INTERNAL_RESET_URL_KEY:
+            self.key = self.request.session.get(INTERNAL_RESET_SESSION_KEY, '')
+            # (Ab)using forms here to be able to handle errors in XHR #890
+            token_form = UserTokenForm(
+                data={'uidb36': uidb36, 'key': self.key})
+            if token_form.is_valid():
+                self.reset_user = token_form.reset_user
+                return super(PasswordResetFromKeyView, self).dispatch(request,
+                                                                      uidb36,
+                                                                      self.key,
+                                                                      **kwargs)
         else:
-            self.reset_user = token_form.reset_user
-            return super(PasswordResetFromKeyView, self).dispatch(request,
-                                                                  uidb36,
-                                                                  key,
-                                                                  **kwargs)
+            token_form = UserTokenForm(
+                data={'uidb36': uidb36, 'key': self.key})
+            if token_form.is_valid():
+                # Store the key in the session and redirect to the
+                # password reset form at a URL without the key. That
+                # avoids the possibility of leaking the key in the
+                # HTTP Referer header.
+                self.request.session[INTERNAL_RESET_SESSION_KEY] = self.key
+                redirect_url = self.request.path.replace(
+                    self.key, INTERNAL_RESET_URL_KEY)
+                return redirect(redirect_url)
+
+        self.reset_user = None
+        response = self.render_to_response(
+            self.get_context_data(token_fail=True)
+        )
+        return _ajax_response(self.request, response, form=token_form)
+
+    def get_context_data(self, **kwargs):
+        ret = super(PasswordResetFromKeyView, self).get_context_data(**kwargs)
+        ret['action_url'] = reverse(
+            'account_reset_password_from_key',
+            kwargs={'uidb36': self.kwargs['uidb36'],
+                    'key': self.kwargs['key']})
+        return ret
 
     def get_form_kwargs(self):
         kwargs = super(PasswordResetFromKeyView, self).get_form_kwargs()
@@ -601,47 +723,60 @@ class PasswordResetFromKeyView(AjaxCapableProcessFormViewMixin, FormView):
 
     def form_valid(self, form):
         form.save()
-        get_adapter().add_message(self.request,
-                                  messages.SUCCESS,
-                                  'account/messages/password_changed.txt')
+        get_adapter(self.request).add_message(
+            self.request,
+            messages.SUCCESS,
+            'account/messages/password_changed.txt')
         signals.password_reset.send(sender=self.reset_user.__class__,
                                     request=self.request,
                                     user=self.reset_user)
+
+        if app_settings.LOGIN_ON_PASSWORD_RESET:
+            return perform_login(
+                self.request, self.reset_user,
+                email_verification=app_settings.EMAIL_VERIFICATION)
+
         return super(PasswordResetFromKeyView, self).form_valid(form)
 
+
 password_reset_from_key = PasswordResetFromKeyView.as_view()
 
 
 class PasswordResetFromKeyDoneView(TemplateView):
-    template_name = "account/password_reset_from_key_done.html"
+    template_name = (
+        "account/password_reset_from_key_done." +
+        app_settings.TEMPLATE_EXTENSION)
+
 
 password_reset_from_key_done = PasswordResetFromKeyDoneView.as_view()
 
 
 class LogoutView(TemplateResponseMixin, View):
 
-    template_name = "account/logout.html"
+    template_name = "account/logout." + app_settings.TEMPLATE_EXTENSION
     redirect_field_name = "next"
 
     def get(self, *args, **kwargs):
         if app_settings.LOGOUT_ON_GET:
             return self.post(*args, **kwargs)
-        if not self.request.user.is_authenticated():
+        if not is_authenticated(self.request.user):
             return redirect(self.get_redirect_url())
         ctx = self.get_context_data()
         return self.render_to_response(ctx)
 
     def post(self, *args, **kwargs):
         url = self.get_redirect_url()
-        if self.request.user.is_authenticated():
+        if is_authenticated(self.request.user):
             self.logout()
         return redirect(url)
 
     def logout(self):
-        get_adapter().add_message(self.request,
-                                  messages.SUCCESS,
-                                  'account/messages/logged_out.txt')
-        auth_logout(self.request)
+        adapter = get_adapter(self.request)
+        adapter.add_message(
+            self.request,
+            messages.SUCCESS,
+            'account/messages/logged_out.txt')
+        adapter.logout(self.request)
 
     def get_context_data(self, **kwargs):
         ctx = kwargs
@@ -653,20 +788,28 @@ class LogoutView(TemplateResponseMixin, View):
         return ctx
 
     def get_redirect_url(self):
-        return (get_next_redirect_url(self.request,
-                                      self.redirect_field_name)
-                or get_adapter().get_logout_redirect_url(self.request))
+        return (
+            get_next_redirect_url(
+                self.request,
+                self.redirect_field_name) or get_adapter(
+                    self.request).get_logout_redirect_url(
+                        self.request))
+
 
 logout = LogoutView.as_view()
 
 
 class AccountInactiveView(TemplateView):
-    template_name = 'account/account_inactive.html'
+    template_name = (
+        'account/account_inactive.' + app_settings.TEMPLATE_EXTENSION)
+
 
 account_inactive = AccountInactiveView.as_view()
 
 
 class EmailVerificationSentView(TemplateView):
-    template_name = 'account/verification_sent.html'
+    template_name = (
+        'account/verification_sent.' + app_settings.TEMPLATE_EXTENSION)
+
 
 email_verification_sent = EmailVerificationSentView.as_view()
