@@ -1,15 +1,15 @@
 # -*- coding: utf-8 -*-
 from __future__ import unicode_literals
 
-from allauth.socialaccount.tests import create_oauth2_tests
-from allauth.tests import MockedResponse
-from allauth.socialaccount.providers import registry
+from allauth.socialaccount.tests import OAuth2TestsMixin
+from allauth.tests import MockedResponse, TestCase
 
 from .provider import DropboxOAuth2Provider
 
 
-class DropboxOAuth2Tests(create_oauth2_tests(registry.by_id(
-        DropboxOAuth2Provider.id))):
+class DropboxOAuth2Tests(OAuth2TestsMixin, TestCase):
+    provider_id = DropboxOAuth2Provider.id
+
     def get_mocked_response(self):
         return [MockedResponse(200, """{
             "display_name": "Björn Andersson",
@@ -19,7 +19,7 @@ class DropboxOAuth2Tests(create_oauth2_tests(registry.by_id(
                 "given_name": "Björn"
                 },
             "locale": "en",
-            "email": "test@example.se",
+            "email": "test@example.com",
             "uid": 1234567890,
             "email_verified": true,
             "quota_info": {
