@@ -1,9 +1,13 @@
 from __future__ import unicode_literals
 
 import requests
-from allauth.socialaccount.providers.oauth2.views import (OAuth2Adapter,
-                                                          OAuth2LoginView,
-                                                          OAuth2CallbackView)
+
+from allauth.socialaccount.providers.oauth2.views import (
+    OAuth2Adapter,
+    OAuth2CallbackView,
+    OAuth2LoginView,
+)
+
 from .provider import WindowsLiveProvider
 
 
@@ -17,14 +21,14 @@ class WindowsLiveOAuth2Adapter(OAuth2Adapter):
         headers = {'Authorization': 'Bearer {0}'.format(token.token)}
         resp = requests.get(self.profile_url, headers=headers)
 
-#example of whats returned (in python format):
-#{'first_name': 'James', 'last_name': 'Smith',
-# 'name': 'James Smith', 'locale': 'en_US', 'gender': None,
-# 'emails': {'personal': None, 'account': 'jsmith@xyz.net',
-# 'business': None, 'preferred': 'jsmith@xyz.net'},
-# 'link': 'https://profile.live.com/',
-# 'updated_time': '2014-02-07T00:35:27+0000',
-# 'id': '83605e110af6ff98'}
+# example of whats returned (in python format):
+# {'first_name': 'James', 'last_name': 'Smith',
+#  'name': 'James Smith', 'locale': 'en_US', 'gender': None,
+#  'emails': {'personal': None, 'account': 'jsmith@example.com',
+#  'business': None, 'preferred': 'jsmith@example.com'},
+#  'link': 'https://profile.live.com/',
+#  'updated_time': '2014-02-07T00:35:27+0000',
+#  'id': '83605e110af6ff98'}
 
         extra_data = resp.json()
         return self.get_provider().sociallogin_from_response(request,
